#ifndef _cdr_h_
#define _cdr_h_

#define HTTP_MAX_SIZE_DATA  2048

typedef struct http_req_ {
    char buf[HTTP_MAX_SIZE_DATA];
    size_t size;
} http_req;

#define CDR_CDR_GUID_INDEX     0
#define CDR_GUID_INDEX         1
#define CDR_SYS_INDEX          2
#define CDR_PBX_INDEX          3
#define CDR_NODE_INDEX         4
#define CDR_DEVICE_INDEX       5
#define CDR_CN_INDEX           6
#define CDR_E164_INDEX         7
#define CDR_H323_INDEX         8
#define CDR_DIR_INDEX          9
#define CDR_UTC_INDEX         10
#define CDR_LOCAL_INDEX       11
#define CDR_PHYS_INDEX        12
#define CDR_TRANSFERRED_INDEX 13
#define CDR_FORWARDED_INDEX   14
#define CDR_DN_ESCAPED_INDEX  15

#define CDR_NUM_FIELDS        16

#define EVENT_MSG_INDEX        0
#define EVENT_TYPE_INDEX       1
#define EVENT_E164_INDEX       2
#define EVENT_H323_INDEX       3
#define EVENT_CONF_INDEX       4
#define EVENT_CAUSE_INDEX      5
#define EVENT_TIME_INDEX       6
#define EVENT_CDR_ID_INDEX     7
#define EVENT_ORDER_INDEX      8
#define EVENT_DN_INDEX         9

#define EVENTS_NUM_FIELDS     10

#define GROUP_NAME_INDEX       0
#define GROUP_ACT_INDEX        1
#define GROUP_TYPE_INDEX       2
#define GROUP_CDR_ID_INDEX     3

#define GROUPS_NUM_FIELDS      4

#define CDRP_CDR_ID_INDEX      0
#define CDRP_CONN_DUR_INDEX    1
#define CDRP_ALERT_DUR_INDEX   2
#define CDRP_TYPE_INDEX        3
#define CDRP_CALL_FLOW_INDEX   4
#define CDRP_CAUSE_INDEX       5
#define CDRP_CONF_INDEX        6
#define CDRP_GROUPS_INDEX      7
#define CDRP_CALL_DUR_INDEX    8
#define CDRP_BILL_DUR_INDEX    9

#define CDRP_NUM_FIELDS       10

struct _cdrs_info_ {
    // cdr
    char *cdr_values[CDR_NUM_FIELDS];

    // cdrp
    char *cdrp_values[CDRP_NUM_FIELDS];
    char *cdrp_flow;
    char *cdrp_groups;

    // groups
    int num_groups;
    char **groups_values; 
    char **groups_name_escaped; 
    char *groups_query;

    // events
    int num_events;
    char **events_values; 
    char **ev_dn_escaped;
    char **ev_h323_escaped;
    int *search_ldap;
    int *ev_cause;
    int *ev_time;
    char *events_query;
} cdrs_info_t;

struct _cdr {
    sem_t *sem;
    struct _cdrs_info_ **cdrs;
    int num_cdrs;
    const char *grp_prep_query[5];
    const char *ev_prep_query[5];
    ldap_servers_t **ldap_act;
    pthread_mutex_t *ldap_mutex;
} cdr_t;

int init_db_conn(PGconn **conn, FCGX_Request *cgi);
void close_db_conn(PGconn **conn);
int send_cdr_to_db(struct _cdrs_info_ * cdr_data, FCGX_Request *cgi, PGconn * conn);
int parse_cdr(struct _cdr* cdr_data, char *query_string, FCGX_Request *cgi);

void free_cdr_data(struct _cdr * cdr);

#endif
