#ifndef _inno_lib_h_
#define _inno_lib_h_

#include "curl.h"
#include "easy.h"
#include "libxml/parser.h"
#include "libxml/tree.h"
#include "process-text-tables.h"
#include "process-languages.h"
#include "process-strings-utf8.h"

extern int trace_level;

#define TRACE_STD                   1
#define TRACE_DB                    2
#define TRACE_TIME                  4
#define TRACE_CALL_FLOW_TOTAL       8
#define TRACE_CALL_FLOW             16
#define TRACE_PARSE_CFG             32

#define MAX_NUM_THREADS             100

#define LICENSE_VERSION             9

#define PG_CONN_STR                 "host = 127.0.0.1 port = 5432 dbname = innovaphone-reporting user = innovaphone-reporting password = '' sslmode = disable"
#define PG_CDR_PREP_TRANS           "cdrprep"
#define PG_CDR_GUID_PREP_TRANS      "cdrguidprep"
#define PG_CDR_LOCK_PREP_TRANS      "cdrlockprep"
#define PG_CDRP_PREP_TRANS          "cdrpprep"
#define PG_LDAP_ACT_PREP_TRANS      "ldapactprep"
#define PG_LDAP_ALL_PREP_TRANS      "ldapallprep"
#define PG_5EVENTS_PREP_TRANS       "eventsprep5"
#define PG_4EVENTS_PREP_TRANS       "eventsprep4"
#define PG_3EVENTS_PREP_TRANS       "eventsprep3"
#define PG_2EVENTS_PREP_TRANS       "eventsprep2"
#define PG_1EVENTS_PREP_TRANS       "eventsprep1"
#define PG_5GROUPS_PREP_TRANS       "groupsprep5"
#define PG_4GROUPS_PREP_TRANS       "groupsprep4"
#define PG_3GROUPS_PREP_TRANS       "groupsprep3"
#define PG_2GROUPS_PREP_TRANS       "groupsprep2"
#define PG_1GROUPS_PREP_TRANS       "groupsprep1"
#define PG_CDR_QUERY                "INSERT INTO cdrs(cdr_guid, guid, sys, cdr_pbx, cdr_node, device, cn, cdr_e164, cdr_h323, cdr_dir, utc_stamp, local_stamp, cdr_phys, transferred, forwarded, cdr_dn) VALUES ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,to_timestamp($11) AT TIME ZONE 'UTC',to_timestamp($12) AT TIME ZONE 'UTC',$13,$14,$15,$16) RETURNING id"
#define PG_CDR_GUID_QUERY           "SELECT cdr_guid FROM cdrs WHERE cdr_guid = $1"
#define PG_CDR_LOCK_QUERY           "LOCK TABLE cdrs IN SHARE MODE"
#define PG_CDRP_QUERY               "INSERT INTO cdr_properties(cdr_id, conn_duration, alert_duration, cdrp_type, cdrp_call_flow, cause_code, cdrp_conf, cdrp_cdr_id_groups, call_duration, billing_duration) VALUES ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10)"
#define PG_EVENTS_QUERY_BEGIN       "INSERT INTO events(msg, ev_type, ev_e164, ev_h323, conf, cause, time, cdr_id, order_index, ev_dn) VALUES "
#define PG_GROUPS_QUERY_BEGIN       "INSERT INTO groups(grp_name, active, grp_type, cdr_id) VALUES "
#define PG_LDAP_ACT_QUERY           "SELECT server, host, port, password, username, base, number_attrib, name_attrib, pbx, cc, ac, np, ip, el, sn, active, filter FROM ldap_servers WHERE active = TRUE"
#define PG_LDAP_ALL_QUERY           "SELECT server, host, port, password, username, base, number_attrib, name_attrib, pbx, cc, ac, np, ip, el, sn, active, filter FROM ldap_servers"
#define PG_5EVENTS_PREP_QUERY       "INSERT INTO events(msg, ev_type, ev_e164, ev_h323, conf, cause, time, cdr_id, order_index, ev_dn) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10), ($11, $12, $13, $14, $15, $16, $17, $18, $19, $20), ($21, $22, $23, $24, $25, $26, $27, $28, $29, $30), ($31, $32, $33, $34, $35, $36, $37, $38, $39, $40), ($41, $42, $43, $44, $45, $46, $47, $48, $49, $50)"
#define PG_4EVENTS_PREP_QUERY       "INSERT INTO events(msg, ev_type, ev_e164, ev_h323, conf, cause, time, cdr_id, order_index, ev_dn) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10), ($11, $12, $13, $14, $15, $16, $17, $18, $19, $20), ($21, $22, $23, $24, $25, $26, $27, $28, $29, $30), ($31, $32, $33, $34, $35, $36, $37, $38, $39, $40)"
#define PG_3EVENTS_PREP_QUERY       "INSERT INTO events(msg, ev_type, ev_e164, ev_h323, conf, cause, time, cdr_id, order_index, ev_dn) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10), ($11, $12, $13, $14, $15, $16, $17, $18, $19, $20), ($21, $22, $23, $24, $25, $26, $27, $28, $29, $30)"
#define PG_2EVENTS_PREP_QUERY       "INSERT INTO events(msg, ev_type, ev_e164, ev_h323, conf, cause, time, cdr_id, order_index, ev_dn) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10), ($11, $12, $13, $14, $15, $16, $17, $18, $19, $20)"
#define PG_1EVENTS_PREP_QUERY       "INSERT INTO events(msg, ev_type, ev_e164, ev_h323, conf, cause, time, cdr_id, order_index, ev_dn) VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10)"
#define PG_5GROUPS_PREP_QUERY       "INSERT INTO groups(grp_name, active, grp_type, cdr_id) VALUES ($1, $2, $3, $4), ($5, $6, $7, $8), ($9, $10, $11, $12), ($13, $14, $15, $16), ($17, $18, $19, $20)"
#define PG_4GROUPS_PREP_QUERY       "INSERT INTO groups(grp_name, active, grp_type, cdr_id) VALUES ($1, $2, $3, $4), ($5, $6, $7, $8), ($9, $10, $11, $12), ($13, $14, $15, $16)"
#define PG_3GROUPS_PREP_QUERY       "INSERT INTO groups(grp_name, active, grp_type, cdr_id) VALUES ($1, $2, $3, $4), ($5, $6, $7, $8), ($9, $10, $11, $12)"
#define PG_2GROUPS_PREP_QUERY       "INSERT INTO groups(grp_name, active, grp_type, cdr_id) VALUES ($1, $2, $3, $4), ($5, $6, $7, $8)"
#define PG_1GROUPS_PREP_QUERY       "INSERT INTO groups(grp_name, active, grp_type, cdr_id) VALUES ($1, $2, $3, $4)"

#define ALARM_FILE_PREFIX           "/var/www/innovaphone/tmp/alarm"
#define REPORTING_INSTALLED         "/var/www/innovaphone/apps/innovaphone-reporting.xml"
#define LDAP_SERVER_FILE            "/home/innovaphone-reporting/tmp/ldap_changed"

#define CONTENT_TYPE_XML            "Content-type: text/xml\r\n"
#define CONTENT_TYPE_HTML           "Content-Type: text/html\r\n"
#define CONTENT_TYPE_PDF            "Content-type: application/pdf\r\n"
#define CONTENT_TYPE_CSV            "Content-type: text/csv\r\n"
#define CONTENT_ENCODING_DEFLATE    "Content-Encoding: deflate\r\n"
#define CONTENT_LENGTH              "Content-Length: "
#define CONTENT_DISPOS              "Content-Disposition: attachment; filename="
#define INTERNAL_SERVER_ERROR       "<html><head></head><body><h1>500 - Internal Server Error</h1></body></html>\r\n"
#define STATUS_500                  "Status: 500\r\n"
#define STATUS_200                  "Status: 200\r\n"
#define EXPIRES_HEADER              "expires: Friday, 25-Jul-97 00:00:00 GMT\r\n"

#define CFG_CONN_DB                 "[ CONN-DB ]"
#define CFG_QUERY                   "[ QUERY ]"
#define CFG_HREF                    "[ HREF ]"
#define CFG_FORMAT                  "[ FORMAT ]"
#define CFG_SORT_ORDER              "[ SORT-ORDER ]"
#define CFG_SORT_VALUE              "[ SORT-VALUE ]"
#define CFG_GROUP_VALUE             "[ GROUP-VALUE ]"               
#define CFG_LOGO                    "[ LOGO ]"
#define CFG_LOGO_WIDTH              "[ LOGO-WIDTH ]"
#define CFG_FOOTER                  "[ FOOTER ]"               
#define CFG_FOOTER_WIDTH            "[ FOOTER-WIDTH ]"
#define CFG_PRINT                   "[ PRINT ]"
#define CFG_MYPBX                   "[ MYPBX ]"
#define CFG_SERVER                  "[ SERVER ]"
#define CFG_REPORT_FILE             "[ REPORT-FILE ]"
#define CFG_REPORT_LIMIT            "[ REPORT-LIMIT ]"
#define CFG_COL_NAMES               "[ COL-NAMES ]"
#define CFG_TRANSLATION_TAG         "[ TRANSLATION-TAG ]"
#define CFG_TRANSLATION_VALUE       "[ TRANSLATION-VALUE ]"
#define CFG_CN                      "[ CN ]"
#define CFG_HIDE_DIGITS             "[ HIDE-DIGITS ]"
#define CFG_FILTERS                 "[ FILTERS ]"
#define CFG_FILTER_NAME             "[ FILTER-NAME ]"
#define CFG_PBX_OBJECT              "[ PBX-OBJECT ]"
#define CFG_FROM_DATE               "[ FROM ]"
#define CFG_TO_DATE                 "[ TO ]"
#define CFG_FILTER_INFO             "[ FILTER-INFO ]"
#define CFG_DISPLAYED_DURATION      "[ DISPLAYED-DURATION ]"
#define CFG_LANGUAGE                "[ LANGUAGE ]"
#define CFG_ANONYM                  "[ ANONYM ]"

#define ACTION_REPORT               "report"
#define ACTION_MISSED               "missed-calls"
#define ACTION_CALLBACK             "callback"
#define ACTION_CLEAR                "clear"

#define HTTP_RESPONSE_BODY_HEAD  "<html><head></head><body><h1>"
#define HTTP_RESPONSE_BODY_TAIL  "</h1></body></html>\r\n"

#define MAX_TRACE_LENGTH 1024

#define LOG_INNOVAPHONE     1
#define LOG_REPORTING       2
#define LOG_CDRS            3
#define LOG_ALARMS          4
#define LOG_LOG             5

extern const char* errors[];
extern int num_errors;

enum {
    NO_ERROR_IN_CDR, /* 0 */
    /* do not change this */
    ERROR_CDR_PARSING_CDR_XML,
    ERROR_CONNECTING_TO_DB,
    ERROR_SETTING_ENCODING,
    ERROR_PREPARING_CDR_TRANSACTION,
    ERROR_PREPARING_EVENT_TRANSACTION,
    ERROR_PREPARING_GROUP_TRANSACTION,
    ERROR_PREPARING_CDRP_TRANSACTION,
    ERROR_PREPARING_LDAP_ALL_TRANSACTION,
    ERROR_PREPARING_LDAP_ACT_TRANSACTION,
    ERROR_PREPARING_CDR_GUID_TRANSACTION,
    ERROR_PREPARING_CDR_LOCK_TRANSACTION,
    ERROR_DB_BEING_RESTORED,
    ERROR_CDR_ATTRIBUTE_NOT_PRESENT,
    ERROR_CDR_ATTRIBUTE_MSG_TIME_MISSING,
    ERROR_CDR_REL_DISC_EVENT_MISSING,
    ERROR_EXECUTING_BEGIN_CMD,
    ERROR_EXECUTING_INSERT_CDR_QUERY,
    ERROR_EXECUTING_INSERT_EV_QUERY,
    ERROR_EXECUTING_INSERT_GRP_QUERY,
    ERROR_EXECUTING_INSERT_CDRP_QUERY,
    ERROR_EXECUTING_COMMIT_CMD,
    /* needed for alarms */
    ERROR_EXECUTING_QUERY,
    ERROR_DOC_ROOT_SCRIPT_NAME_EMPTY,
    NO_MEMORY_FOR_POST_REQ,
    INCOMPLETE_POST_REQUEST,
    ERROR_EMPTY_QUERY,
    ERROR_CONTENT_LEN_FIELD_MISSING,
    ERROR_UNKNOWN_REQUEST,
    NO_MSG_FIELD_GET_REQ,
    NO_MSG_FIELD_LOG_REQ,
    NO_REPORTING_INSTALLED,
    NO_SCRIPT_FOUND,
    NO_MEMORY_FOR_FILENAME,
    NO_CURL_INSTANCE,
    ERROR_EXECUTING_CMD,
    ERROR_CREATING_OPENING_FILE,
    ERROR_OPENING_LOG_FILE,
    ERROR_OPENING_ALARM_FILE,
    ERROR_OPENING_REL_CDR_FILE,
    ERROR_WRITING_LOG_FILE,
    ERROR_WRITING_ALARM_FILE,
    ERROR_WRITING_REL_CDR_FILE,
    ERROR_MYPBX_PARAMETER_NOT_SET,
    ERROR_MYPBX_ACTION_NOT_DEFINED,
    ERROR_MYPBX_CHALLENGE_NOT_CREATED,
    ERROR_MYPBX_CHALLENGE_NOT_VALID,
    ERROR_MYPBX_CN_NOT_MATCHING_CHALLENGE,
    ERROR_MYPBX_AUTH_NOT_SET,
    ERROR_MYPBX_WRONG_AUTH_RECV,
    ERROR_MYPBX_FETCHING_MISSED_CALLS,
    ERROR_MYPBX_CB_PARAMS_NOT_SET,
    ERROR_MYPBX_CB_UPDATE_INFO_FAILED,
    ERROR_MYPBX_CREATING_TMP_FILE,
    ERROR_REPORT_NO_CONFIG_FILE,
    ERROR_REPORT_OPENING_CFG,
    ERROR_REPORT_PARSING_CFG_FILE,
    ERROR_REPORT_NO_MEMORY_AVAILABLE,
    ERROR_REPORT_LOADING_IMAGES,
    ERROR_REPORT_CALCULATE_DIM,
    ERROR_REPORT_OPENING_ZIP_FILE,
    ERROR_REPORT_CREATING_PDF_OBJECT,
    ERROR_REPORT_CREATING_NEW_PDF_DOC,
    ERROR_MYPBX_UPDATING_CLEAR_REPORT,
    ERROR_MYPBX_INSERTING_CLEAR_REPORT,
    ERROR_EXECUTING_LOCK_CDR_CMD,
    ERROR_UNLICENSED_CDR,
    ERROR_INSERTING_CDR,
    ERROR_LDAP_SERVER_READ,
    ERROR_DB_HANDLER_NOT_VALID /* 66 */
};

#define MAX_NUM_ATTRIBUTES 8

typedef struct alarm_server_ {
    char * ip;
    char * password;
    char * user;
    char * port;
    int https;
} alarm_server_t;

typedef struct inno_config_ {
    char * error;
    alarm_server_t * alarm;
} inno_config_t;

void cleanup_inno_config(inno_config_t * cfg);
inno_config_t * read_inno_config();

char * str_malloc(const char * from);
char * rc4_decrypt(char * enc_text, char * key);
int hexstr_to_bytestr(char * hex, unsigned char* byte);
void strrev(char * str);
char * xmlNodeGetContentChar(xmlNodePtr node);
char * xml_entities(char * inp);
void do_log(int log_type, char * application, char * format, ...);
char * get_url_param(CURL * conn, char * query_string, char * name, char * from, int log_type);
int timeval_subtract(struct timeval *result, struct timeval *t2, struct timeval *t1);
int read_trace_level();
int lookup_language(char * lang);
void strcat_xml(char * out, const char * inp);
int check_db_restore(void);

#endif
